@@ -1,7 +1,7 @@
 
 /*  $Header$
  *
- *  Copyright (c) 1987-2007 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2008 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,64 +41,94 @@ static double max_fix[MAXBASES] = {
 
 static char *make_eng_sci(int *, int);
 
-/* Add in the thousand separators characters if required and if we are
- * currently in the decimal numeric base, use the "right" radix character.
- */
-
+/* Add in the thousand separators characters if required */
 void
-localize_number(char *dest, const char *src)
+localize_expression(char *dest, const char *src, int dest_length, int *cursor)
 {
-    char tnum[MAX_LOCALIZED], *dstp;
-
-    if (!v->error && v->show_tsep && v->base == DEC) {
-        const char *radixp, *srcp;
-        int n, i;
-        size_t tsep_len;
-
-        /* Process the fractional part (if any). */
-        srcp = src + strlen(src) - 1;
-        dstp = tnum;
-        if ((radixp = strchr(src, '.')) != NULL) {
-            while (srcp != radixp) {
-                *dstp++ = *srcp--;
-            }
-            *dstp++ = *srcp--;
-        }
-
-        /* Process the integer part, add in thousand separators. */
-        tsep_len = strlen(v->tsep);
-        n = 0;
-        while (srcp >= src) {
-            *dstp++ = *srcp--;
-            n++;
-            if (n == 3 && srcp >= src && *srcp != '-') {
-                for (i = tsep_len - 1; i >= 0; i--) {
-                    *dstp++ = v->tsep[i];
-                }
-                n = 0;
-            }
-        }
-        *dstp++ = '\0';
-
-        /* Move from scratch pad to fnum, reversing the character order. */
-        srcp = tnum + strlen(tnum) - 1;
-        dstp = dest;
-        while (srcp >= tnum) {
-            *dstp++ = *srcp--;
-        }
-        *dstp++ = '\0';
+    GString *clean, *output;
+    const char *c, *d;
+    int digit_count = -1, read_cursor, new_cursor;
+    gboolean after_radix = FALSE;
+    
+    /* Only modify if valid */
+    if (v->error || v->base != DEC) {
+        STRNCPY(dest, src, dest_length - 1);
+        return;
+    }
+    
+    if (cursor) {
+        new_cursor = *cursor;
     } else {
-        STRCPY(dest, src);
+        new_cursor = -1;
     }
-    dstp = strchr(dest, '.');
-    if (dstp != NULL) {
-        size_t radix_len;
 
-        radix_len = strlen(v->radix);
-        if (radix_len != 1) {
-            memmove(dstp + radix_len, dstp + 1, strlen (dstp + 1) + 1);
+    /* Remove separators if not supported */
+    clean = g_string_sized_new(strlen(src));
+    for (c = src, read_cursor = 1; *c; c++, read_cursor++) {
+        if (v->tsep[0] != '\0' && strncmp(c, v->tsep, strlen(v->tsep)) == 0) {
+            c += strlen(v->tsep) - 1;
+            if (new_cursor >= read_cursor) {
+                new_cursor--;
+            }
+            read_cursor--;
         }
-        MEMCPY(dstp, v->radix, radix_len);
+        else {
+            g_string_append_c(clean, *c);
+        }
+    }
+
+    if (!v->show_tsep) {
+        STRNCPY(dest, clean->str, dest_length - 1);
+        g_string_free(clean, TRUE);
+        return;
+    }
+
+    /* Scan expression looking for numbers and inserting separators */
+    output = g_string_sized_new(dest_length);
+    for (c = clean->str, read_cursor = 1; *c; c++, read_cursor++) {
+        /* Insert separators between digits */
+        if (*c >= '0' && *c <= '9') {
+            /* Read ahead to find the number of digits */
+            if (digit_count < 0) {
+                digit_count = 1;
+                for (d = c + 1; *d >= '0' && *d <= '9'; d++) {
+                    digit_count++;
+                }
+            }
+            
+            g_string_append_c(output, *c);
+            
+            /* Insert separator after nth digit */
+            if (!after_radix && digit_count > 1 && digit_count % v->tsep_count == 1) {
+                g_string_append(output, v->tsep);
+                if (new_cursor > read_cursor) {
+                    new_cursor++;
+                }
+                read_cursor++;
+            }
+            digit_count--;
+        }
+        /* Ignore digits after the radix */
+        else if (strncmp(c, v->radix, strlen(v->radix)) == 0) {
+            digit_count = -1;
+            after_radix = TRUE;
+            c += strlen(v->radix) - 1;
+            g_string_append(output, v->radix);
+        }
+        /* Reset when encountering other characters (e.g. '+') */
+        else {
+            digit_count = -1;
+            after_radix = FALSE;
+            g_string_append_c(output, *c);
+        }
+    }
+    
+    STRNCPY(dest, output->str, dest_length - 1);
+    g_string_free(output, TRUE);
+    g_string_free(clean, TRUE);
+    
+    if (cursor != NULL && *cursor != -1) {
+        *cursor = new_cursor;
     }
 }
 
@@ -168,11 +198,11 @@ initialise()
 char *
 make_fixed(int *MPnumber, char *str, int base, int cmax, int toclear)
 {
-    char half[4], *optr;
+    char half[MAXLINE], *optr;
     int MP1base[MP_SIZE], MP1[MP_SIZE], MP2[MP_SIZE], MPval[MP_SIZE];
     int ndig;                   /* Total number of digits to generate. */
     int ddig;                   /* Number of digits to left of decimal sep. */
-    int dval, n;
+    int dval, n, i;
  
     optr = str;
     mpabs(MPnumber, MPval);
@@ -186,7 +216,7 @@ make_fixed(int *MPnumber, char *str, int base, int cmax, int toclear)
 
     mppwr(MP1base, &v->accuracy, MP1);
     /* FIXME: string const. if MPstr_to_num can get it */
-    SPRINTF(half, "0.5");
+    SNPRINTF(half, MAXLINE, "0.5");
     MPstr_to_num(half, DEC, MP2);
     mpdiv(MP2, MP1, MP1);
     mpadd(MPval, MP1, MPval);
@@ -207,7 +237,8 @@ make_fixed(int *MPnumber, char *str, int base, int cmax, int toclear)
 
     while (ndig-- > 0) {
         if (ddig-- == 0) {
-            *optr++ = '.';
+            for (i = 0; i < strlen(v->radix); i++)
+                *optr++ = v->radix[i];
         }
         mpmul(MPval, MP1base, MPval);
         mpcmi(MPval, &dval);
@@ -226,15 +257,16 @@ make_fixed(int *MPnumber, char *str, int base, int cmax, int toclear)
     }
     v->pointed = 0;
 
-    if (!v->show_zeroes && v->accuracy != 0) {
-        optr = str + strlen(str) - 1;
-        while (*optr == '0') {
-            optr--;
+    /* Strip off trailing zeroes */
+    if (!v->show_zeroes) {
+        for (i = strlen(str) - 1; i > 1 && str[i] == '0'; i--) {
+            str[i] = '\0';
+        }
+        
+        /* If no fractional part discard radix */
+        if (strlen(str) >= strlen(v->radix) && strcmp(str + strlen(str) - strlen(v->radix), v->radix) == 0) {
+            str[strlen(str) - strlen(v->radix)] = '\0';
         }
-    if (optr < str || *optr != '.') {
-        optr++;
-    }
-        *optr = '\0';
     }
 
     return(str);
@@ -276,7 +308,7 @@ make_number(int *MPnumber, int base, int ignoreError)
 static char *
 make_eng_sci(int *MPnumber, int base)
 {
-    char half[4], fixed[MAX_DIGITS], *optr;
+    char half[MAXLINE], fixed[MAX_DIGITS], *optr;
     int MP1[MP_SIZE], MPatmp[MP_SIZE], MPval[MP_SIZE];
     int MP1base[MP_SIZE], MP3base[MP_SIZE], MP10base[MP_SIZE];
     int i, dval, len, n;
@@ -335,7 +367,7 @@ make_eng_sci(int *MPnumber, int base)
         }
     }
  
-    STRCPY(fixed, make_fixed(MPmant, v->fnum, base, MAX_DIGITS-6, TRUE));
+    STRNCPY(fixed, make_fixed(MPmant, v->fnum, base, MAX_DIGITS-6, TRUE), MAX_DIGITS - 1);
     len = strlen(fixed);
     for (i = 0; i < len; i++) {
         *optr++ = fixed[i];
@@ -350,7 +382,7 @@ make_eng_sci(int *MPnumber, int base)
         *optr++ = '+';
     }
  
-    SPRINTF(half, "0.5");
+    SNPRINTF(half, MAXLINE, "0.5");
     MPstr_to_num(half, DEC, MP1);
     mpaddi(MP1, &exp, MPval);
     n = 1;
@@ -570,7 +602,7 @@ void
 refresh_display(int cursor)
 {
     int i, MP_reg[MP_SIZE];
-    char localized[MAX_LOCALIZED], *str, *ans, reg[3];
+    char localized[MAX_LOCALIZED], *str, *ans, reg[3], *t;
     struct exprm_state *e;
 
     switch (v->syntax) {
@@ -588,16 +620,20 @@ refresh_display(int cursor)
             }
             ans = make_number(e->ans, v->base, TRUE);
 
-            localize_number(localized, ans);
-            str_replace(&str, "Ans", localized);
+            localize_expression(localized, ans, MAX_LOCALIZED, &cursor);
+            t = str_replace(str, "Ans", localized);
+            free(str);
+            str = t;
 
             /* Replace registers with values. */
             for (i = 0; i < 10; i++) {
                 SNPRINTF(reg, 3, "R%d", i);
                 do_rcl_reg(i, MP_reg);
-                str_replace(&str, reg, make_number(MP_reg, v->base, FALSE));
+                t = str_replace(str, reg, make_number(MP_reg, v->base, FALSE));
+                free(str);
+                str = t;
             }
-
+        
             ui_set_display(str, cursor);
             free(str);
             break;
