@@ -205,7 +205,7 @@ peek_previous_state(void)
 }
 
 static int
-is_undo_step()
+is_undo_step(void)
 {
     return(v->h.current != v->h.begin);
 }
@@ -237,7 +237,7 @@ do_accuracy(int value)     /* Set display accuracy. */
 
 
 void
-do_business()     /* Perform special business mode calculations. */
+do_business(void)     /* Perform special business mode calculations. */
 {
     if (v->current == KEY_FINC_CTRM) {
         calc_ctrm(v->MPdisp_val);
@@ -282,7 +282,7 @@ exp_insert(char *text, int cursor)
 
 
 void
-exp_clear()
+exp_clear(void)
 {
     exp_replace("");
 }
@@ -396,7 +396,7 @@ exp_replace(char *text)
 
 
 static void
-exp_negate()
+exp_negate(void)
 {
     struct exprm_state *e = get_state();
 
@@ -414,7 +414,7 @@ exp_negate()
 
 
 static void
-exp_inv()
+exp_inv(void)
 {
     struct exprm_state *e = get_state();
 
@@ -557,9 +557,26 @@ do_expression(int function, int arg, int cursor)
             /* Solve the equation */
             } else {
                 int MPval[MP_SIZE];
-                char *message = NULL;
+                char *c, *message = NULL;
+                GString *clean;
+                int result;
                 
-                switch (ce_parse(e->expression, MPval)) {
+                /* Remove thousands separators and use english radix */
+                clean = g_string_sized_new(strlen(e->expression));
+                for (c = e->expression; *c; c++) {
+                    if (v->tsep[0] != '\0' && strncmp(c, v->tsep, strlen(v->tsep)) == 0) {
+                        c += strlen(v->tsep) - 1;
+                    } else if (strncmp(c, v->radix, strlen(v->radix)) == 0) {
+                        g_string_append_c(clean, '.');
+                        c += strlen(v->radix) - 1;
+                    } else {
+                        g_string_append_c(clean, *c);
+                    }
+                }
+                result = ce_parse(clean->str, MPval);
+                g_string_free(clean, TRUE);
+                
+                switch (result) {
                     case 0:
                         mpstr(MPval, e->ans);
                         exp_replace("Ans");
@@ -632,7 +649,7 @@ do_expression(int function, int arg, int cursor)
 
 
 void
-do_calc()      /* Perform arithmetic calculation and display result. */
+do_calc(void)      /* Perform arithmetic calculation and display result. */
 {
     double dval, dres;
     int MP1[MP_SIZE], MP2[MP_SIZE];
@@ -875,7 +892,7 @@ do_percent(void)
 
 /* Clear the calculator display and re-initialise. */
 void
-do_clear()
+do_clear(void)
 {
     clear_display(TRUE);
     if (v->error) {
@@ -887,7 +904,7 @@ do_clear()
 
 /* Clear the calculator display. */
 void
-do_clear_entry()
+do_clear_entry(void)
 {
     clear_display(FALSE);
 }
@@ -952,7 +969,7 @@ do_constant(int index)
 
 /* Remove the last numeric character typed. */
 void
-do_backspace()
+do_backspace(void)
 {
     size_t len;
 
@@ -987,7 +1004,7 @@ do_backspace()
 
 
 void
-do_delete()
+do_delete(void)
 {
     /* Not required in ltr mode */
 }
@@ -1036,7 +1053,7 @@ do_exchange(int index)
 
 /* Get exponential number. */
 void
-do_expno()
+do_expno(void)
 {
     v->pointed = (strchr(v->display, '.') != NULL);
     if (!v->new_input) {
@@ -1222,7 +1239,7 @@ do_immedfunc(int s[MP_SIZE], int t[MP_SIZE])
 
 
 void
-do_immed()
+do_immed(void)
 {
     do_immedfunc(v->MPdisp_val, v->MPdisp_val);
     show_display(v->MPdisp_val);
@@ -1230,7 +1247,7 @@ do_immed()
 
 
 void
-do_number()
+do_number(void)
 {
     int offset;
 
@@ -1287,7 +1304,7 @@ do_numtype(enum num_type n)   /* Set number display type. */
 
 
 void
-do_paren()
+do_paren(void)
 {
     ui_set_statusbar("", "");
 
@@ -1398,7 +1415,7 @@ do_rcl_reg(int reg, int value[MP_SIZE])
 
 
 void
-syntaxdep_show_display()
+syntaxdep_show_display(void)
 {
     switch (v->syntax) {
         case NPA:
@@ -1416,7 +1433,7 @@ syntaxdep_show_display()
 
 
 void
-do_point()                   /* Handle numeric point. */
+do_point(void)                   /* Handle numeric point. */
 {
     if (!v->pointed) {
         if (v->toclear) {
@@ -1457,7 +1474,7 @@ do_portionfunc(int num[MP_SIZE])
 
 
 void
-do_portion()
+do_portion(void)
 {
     do_portionfunc(v->MPdisp_val);
     show_display(v->MPdisp_val);
