@@ -218,13 +218,20 @@ get_tsep()
     }
 
     if (tsep == NULL) {
-        return(",");
+        return("");
     } else {
         return(tsep);
     }
 }
 
 
+int
+get_tsep_count()
+{
+    return 3;
+}
+
+
 void
 read_resources()    /* Read all possible resources from the database. */
 {
